'use strict';

var fs = require('fs');
var path = require('path');
var chalk = require('chalk');
var yargs = require('yargs');
var remark = require('remark');
var toc = require('remark-toc');
var typedoc = require('typedoc');
var typescript = require('typescript');
var reactDocgenTypescript = require('react-docgen-typescript');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var remark__default = /*#__PURE__*/_interopDefaultLegacy(remark);
var toc__default = /*#__PURE__*/_interopDefaultLegacy(toc);

const traverseFolder = (filePath, levels = 10, fileName = 'package.json') => {
    const files = fs__default['default'].readdirSync(filePath);
    if (levels === 0) {
        return null;
    }
    const pckg = files.find(file => file === fileName);
    if (pckg) {
        return path__default['default'].resolve(filePath, pckg);
    }
    return traverseFolder(path__default['default'].resolve(filePath, '..'), levels - 1, fileName);
};
const getRepoPath = (initialPath = './') => {
    const packageFileName = traverseFolder(initialPath);
    if (packageFileName) {
        const packageJSON = JSON.parse(fs__default['default'].readFileSync(packageFileName, 'utf8'));
        const { repository } = packageJSON;
        const url = typeof repository === 'string' ? repository : repository.url;
        const { directory } = repository;
        if (url) {
            const baseName = url
                .split('.')
                .slice(0, -1)
                .join('.');
            return {
                repo: `${baseName}${directory ? `/tree/master/${directory}` : ''}`,
                filePath: packageFileName,
            };
        }
    }
    return {};
};

const cleanQuotes = (txt) => txt ? txt.replace(/['"]+/g, '') : txt;
const extractCustomTag = (node, tagName) => {
    const nodes = node.children &&
        node.children.filter((child) => child.type === 'html' &&
            child.value &&
            child.value.startsWith(`<${tagName}`));
    return nodes
        ? nodes
            .map((section) => ({
            section,
            match: section.value
                ? section.value.match(/(\w+)\s*=\s*((["'])(.*?)\3|([^>\s]*)(?=\s|\/>))(?=[^<]*>)/g)
                : undefined,
        }))
            .map(({ section, match }) => ({
            attrs: { section, tagName, node },
            attributes: match
                ? match.map((m) => m.split('=').map(value => cleanQuotes(value)))
                : undefined,
        }))
        : undefined;
};
const inlineNewContent = ({ section, tagName, node }, newContent) => {
    const startTag = `<!-- START-${tagName.toUpperCase()} -->`;
    const endTag = `<!-- END-${tagName.toUpperCase()} -->`;
    const index = node.children ? node.children.indexOf(section) : -1;
    if (index === -1) {
        return;
    }
    let deleteNodes = 0;
    let foundComment = false;
    if (node.children) {
        for (let i = index + 1; i < node.children.length; i += 1) {
            const childNode = node.children[i];
            if (childNode.type == 'html' && childNode.value == startTag) {
                foundComment = true;
            }
            if (foundComment) {
                deleteNodes += 1;
            }
            if (childNode &&
                childNode.type == 'html' &&
                (childNode.value == endTag ||
                    (childNode.value &&
                        childNode.value.toLowerCase().startsWith(`<${tagName}`)))) {
                break;
            }
        }
        node.children.splice(index + 1, deleteNodes, ...[
            {
                type: 'html',
                value: startTag,
            },
            ...newContent,
            {
                type: 'html',
                value: endTag,
            },
        ]);
    }
};
const traverseDirs = (attributes, callback) => {
    const getDirectories = (folder, excludeFiles, repoDir, newNodes) => {
        const items = fs__default['default']
            .readdirSync(folder, { withFileTypes: true })
            .filter(entry => !excludeFiles.some(excluded => entry.name.match(new RegExp(excluded))));
        const files = items.filter(entry => !entry.isDirectory());
        for (const entry of files) {
            callback(entry.name, path__default['default'].join(folder, entry.name), repoDir);
        }
        items
            .filter(entry => entry.isDirectory())
            .forEach(entry => getDirectories(path__default['default'].join(folder, entry.name), excludeFiles, `${repoDir}/${entry.name}`));
    };
    if (attributes) {
        const file = attributes.find(attribute => attribute[0] === 'path');
        const sourcePath = file ? file[1] : './src';
        const { repo = '' } = getRepoPath(sourcePath) || {};
        const excludeFiles = attributes.find(attribute => attribute[0] === 'exclude');
        const url = new URL(repo);
        getDirectories(path__default['default'].resolve(sourcePath), excludeFiles ? excludeFiles[1].split(',') : ['index.ts'], `${url.origin}${path__default['default'].join(url.pathname, sourcePath)}`);
    }
};

const insertOverview = () => {
    return (node) => {
        const sections = extractCustomTag(node, 'package-section');
        if (sections) {
            sections.forEach(({ attrs, attributes }) => {
                if (attributes) {
                    const file = attributes.find(attribute => attribute[0] === 'file');
                    const sectionAttr = attributes.find(attribute => attribute[0] === 'section');
                    const sectionName = sectionAttr
                        ? sectionAttr[1].toLowerCase()
                        : 'overview';
                    if (file) {
                        const content = fs__default['default'].readFileSync(file[1], 'utf8');
                        const overviewNodes = [];
                        const packageJSONFilename = path__default['default'].resolve(path__default['default'].join(path__default['default'].dirname(file[1]), 'package.json'));
                        if (fs__default['default'].existsSync(packageJSONFilename)) {
                            const packageJSON = JSON.parse(fs__default['default'].readFileSync(packageJSONFilename, 'utf8'));
                            const { name, description, repository: { directory }, } = packageJSON;
                            const rootRepoDir = `https://github.com/ccontrols/component-controls/blob/master/${directory}`;
                            overviewNodes.push({
                                type: 'heading',
                                depth: 2,
                                children: [
                                    {
                                        type: 'link',
                                        url: rootRepoDir,
                                        children: [{ type: 'text', value: name }],
                                    },
                                ],
                            });
                            overviewNodes.push({
                                type: 'paragraph',
                                children: [{ type: 'text', value: description }],
                            });
                        }
                        remark__default['default']()
                            .use(() => (sectionNode) => {
                            let foundDepth = 0;
                            if (sectionNode.children) {
                                for (const section of sectionNode.children) {
                                    if (section.type === 'heading') {
                                        if (section.children &&
                                            section.children.length > 0 &&
                                            section.children[0].type === 'text' &&
                                            section.children[0].value &&
                                            section.children[0].value.toLowerCase() === sectionName) {
                                            foundDepth = section.depth;
                                        }
                                        else if (foundDepth >= section.depth) {
                                            break;
                                        }
                                        else if (foundDepth > 0) {
                                            overviewNodes.push(section);
                                        }
                                    }
                                    else if (foundDepth > 0) {
                                        overviewNodes.push(section);
                                    }
                                }
                            }
                        })
                            .process(content);
                        inlineNewContent(attrs, overviewNodes);
                    }
                }
            });
        }
    };
};

const createPropsRow = ({ name, isOptional, type, description, }) => {
    return {
        type: 'tableRow',
        children: [
            {
                type: 'tableCell',
                children: [
                    { type: 'inlineCode', value: `${name}${isOptional ? '' : '*'}` },
                ],
            },
            {
                type: 'tableCell',
                children: type,
            },
            {
                type: 'tableCell',
                children: [{ type: 'text', value: description || '' }],
            },
        ],
    };
};
const createPropsTable = (title, children) => {
    const propsTable = [];
    let table = undefined;
    if (children) {
        propsTable.push({
            type: 'paragraph',
            children: [
                {
                    type: 'heading',
                    depth: 3,
                    children: [
                        {
                            type: 'text',
                            value: title,
                        },
                    ],
                },
            ],
        });
        table = {
            type: 'table',
            children: [
                {
                    type: 'tableRow',
                    children: [
                        { type: 'tableCell', children: [{ type: 'text', value: 'Name' }] },
                        { type: 'tableCell', children: [{ type: 'text', value: 'Type' }] },
                        {
                            type: 'tableCell',
                            children: [{ type: 'text', value: 'Description' }],
                        },
                    ],
                },
            ],
        };
        propsTable.push({
            type: 'paragraph',
            children: [table],
        });
        // eslint-disable-next-line prefer-spread
        table.children.push(...children.map((child) => createPropsRow(child)));
    }
    return { propsTable, table };
};

/* eslint-disable prefer-spread */
const app = new typedoc.Application();
app.bootstrap({
    mode: 'modules',
    logger: 'console',
    target: typescript.ScriptTarget.ES5,
    module: typescript.ModuleKind.CommonJS,
    experimentalDecorators: true,
    includeDeclarations: true,
    excludeExternals: true,
    excludePrivate: true,
    esModuleInterop: true,
});
const extractTSDoc = (files, entries, linkMaps) => {
    const unresolvedTypeNames = [];
    const addedTypeNames = [];
    const repoNames = {};
    const extractPropTable = (nodes, title) => {
        const props = nodes &&
            nodes.map &&
            nodes.reduce((acc, child) => {
                if (child.declaration) {
                    if (!child.declaration.children) {
                        return acc;
                    }
                    return [
                        ...acc,
                        ...child.declaration.children.map((d) => ({
                            name: d.name,
                            isOptional: d.flags ? d.flags.isOptional : true,
                            type: d.type
                                ? extractPropType(d.type)
                                : extractFunction(d, false),
                            description: d.comment ? d.comment.shortText : '',
                        })),
                    ];
                }
                else {
                    return [
                        ...acc,
                        {
                            name: child.name,
                            isOptional: child.flags ? child.flags.isOptional : true,
                            type: child.type
                                ? extractPropType(child.type.type ? child.type : child)
                                : extractFunction(child, false),
                            description: child.comment ? child.comment.shortText : '',
                        },
                    ];
                }
            }, []);
        return createPropsTable(title, props);
    };
    const extractFunction = (node, extractTable = true) => {
        const result = [];
        const declaration = {
            type: 'paragraph',
            children: [],
        };
        addedTypeNames.push(node.name);
        result.push(declaration);
        declaration.children.push({
            type: 'strong',
            children: [
                {
                    type: 'text',
                    value: 'function',
                },
            ],
        });
        declaration.children.push({
            type: 'text',
            value: ' ',
        });
        if (node.kindString !== 'Type literal') {
            declaration.children.push({
                type: 'text',
                value: node.name,
            });
        }
        const signature = node.signatures &&
            node.signatures.find((s) => s.kindString === 'Call signature');
        if (signature) {
            declaration.children.push({
                type: 'text',
                value: '(',
            });
            if (signature.parameters) {
                for (let i = 0; i < signature.parameters.length; i += 1) {
                    const p = signature.parameters[i];
                    if (i > 0) {
                        declaration.children.push({
                            type: 'text',
                            value: ', ',
                        });
                    }
                    declaration.children.push({
                        type: 'inlineCode',
                        value: p.name,
                    });
                    if (!p.flags.isOptional) {
                        declaration.children.push({
                            type: 'text',
                            value: '*',
                        });
                    }
                    declaration.children.push({
                        type: 'text',
                        value: ': ',
                    });
                    declaration.children.push(...extractPropType(p.type));
                }
            }
            declaration.children.push({
                type: 'text',
                value: ')',
            });
            if (signature.type) {
                declaration.children.push({
                    type: 'text',
                    value: ': ',
                });
                declaration.children.push(...extractPropType(signature.type));
            }
            declaration.children.push({
                type: 'text',
                value: ';',
            });
            if (extractTable) {
                const { propsTable, table } = extractPropTable(signature.parameters, 'parameters');
                if (table && signature.type) {
                    table.children.push(createPropsRow({
                        name: 'returns',
                        isOptional: true,
                        type: extractPropType(signature.type),
                        description: signature.comment ? signature.comment.returns : '',
                    }));
                }
                result.push(...propsTable);
            }
        }
        return result;
    };
    const extractInterface = (node) => {
        const result = [];
        const declaration = {
            type: 'paragraph',
            children: [],
        };
        addedTypeNames.push(node.name);
        result.push(declaration);
        if (node.extendedTypes && node.extendedTypes.length) {
            declaration.children.push({
                type: 'strong',
                children: [
                    {
                        type: 'text',
                        value: ' extends ',
                    },
                    ...node.extendedTypes.reduce((acc, t) => [...acc, ...extractPropType(t)], []),
                ],
            });
        }
        const children = node.indexSignature;
        if (children) {
            children.forEach((signature) => {
                for (let i = 0; i < signature.parameters.length; i += 1) {
                    const p = signature.parameters[i];
                    if (i > 0) {
                        declaration.children.push({
                            type: 'text',
                            value: ', ',
                        });
                    }
                    declaration.children.push({
                        type: 'inlineCode',
                        value: p.name,
                    });
                    if (!p.flags.isOptional) {
                        declaration.children.push({
                            type: 'text',
                            value: '*',
                        });
                    }
                    declaration.children.push({
                        type: 'text',
                        value: ': ',
                    });
                    declaration.children.push(...extractPropType(p.type));
                }
                if (signature.type) {
                    declaration.children.push({
                        type: 'text',
                        value: ': ',
                    });
                    declaration.children.push(...extractPropType(signature.type));
                }
            });
        }
        const { propsTable } = extractPropTable(node.children || node.type, 'properties');
        result.push(...propsTable);
        return result;
    };
    const extractPropType = (p, extractTable = false) => {
        switch (p.type) {
            case 'reference': {
                if (p.typeArguments) {
                    const typeArguments = p.typeArguments.reduce((acc, a, idx) => {
                        const r = [...acc, ...extractPropType(a)];
                        if (idx < p.typeArguments.length - 1) {
                            r.push({
                                type: 'text',
                                value: ', ',
                            });
                        }
                        return r;
                    }, []);
                    return [
                        {
                            type: 'text',
                            value: p.name,
                        },
                        {
                            type: 'text',
                            value: '<',
                        },
                        ...typeArguments,
                        {
                            type: 'text',
                            value: '>',
                        },
                    ];
                }
                if (unresolvedTypeNames.indexOf(p.name) === -1) {
                    unresolvedTypeNames.push(p.name);
                }
                const nodeArguments = [];
                if (p.nodeArguments) {
                    p.nodeArguments.forEach((arg) => nodeArguments.push(...extractPropType(arg)));
                }
                return [
                    {
                        type: 'link',
                        url: linkMaps[p.name] || `#${p.name.toLowerCase()}`,
                        children: [
                            {
                                type: 'text',
                                value: p.name,
                            },
                        ],
                    },
                    ...nodeArguments,
                ];
            }
            case 'reflection': {
                if (p.declaration.signatures) {
                    if (p.declaration.signatures.type) {
                        return extractPropType(p.declaration.signatures.type);
                    }
                    else {
                        return extractFunction(p.declaration, extractTable);
                    }
                }
                if (p.declaration.indexSignature &&
                    p.declaration.indexSignature.length) {
                    const signature = p.declaration.indexSignature[0];
                    const children = signature.parameters.reduce((acc, p) => {
                        const r = [
                            ...acc,
                            { type: 'text', value: p.name },
                            { type: 'text', value: ': ' },
                            ...extractPropType(p.type),
                        ];
                        return r;
                    }, []);
                    // console.log(children);
                    return [
                        {
                            type: 'text',
                            value: '[',
                        },
                        ...children,
                        {
                            type: 'text',
                            value: ']',
                        },
                        {
                            type: 'text',
                            value: ': ',
                        },
                        {
                            type: 'paragraph',
                            children: extractPropType(signature.type),
                        },
                    ];
                }
                if (!p.declaration.children) {
                    return [];
                }
                return p.declaration.children
                    .map((t) => ({ t, value: extractPropType(t.type) }))
                    .reduce((acc, { t, value }) => {
                    const children = [];
                    if (t.comment) {
                        children.push({
                            type: 'paragraph',
                            children: [
                                {
                                    type: 'text',
                                    value: t.comment.shortText,
                                },
                            ],
                        });
                    }
                    children.push({
                        type: 'paragraph',
                        children: [
                            {
                                type: 'strong',
                                children: [
                                    {
                                        type: 'text',
                                        value: t.name,
                                    },
                                ],
                            },
                            {
                                type: 'text',
                                value: ': ',
                            },
                            ...value,
                        ],
                    });
                    return [...acc, ...children];
                }, []);
            }
            case 'intersection': {
                const { propsTable } = extractPropTable(p.types, 'properties');
                return propsTable;
            }
            case 'array': {
                return [
                    {
                        type: 'paragraph',
                        children: [
                            ...extractPropType(p.elementType),
                            {
                                type: 'text',
                                value: '[]',
                            },
                        ],
                    },
                ];
            }
            case 'union': {
                return [
                    {
                        type: 'paragraph',
                        children: p.types.reduce((acc, t, idx) => {
                            const r = [...acc, ...extractPropType(t)];
                            if (idx < p.types.length - 1) {
                                r.push({ type: 'text', value: ' | ' });
                            }
                            return r;
                        }, []),
                    },
                ];
            }
            case 'tuple': {
                return p.elements
                    ? [
                        {
                            type: 'paragraph',
                            children: [
                                { type: 'text', value: '[' },
                                ...p.elements.reduce((acc, t, idx) => {
                                    const r = extractPropType(t);
                                    if (idx < p.elements.length - 1) {
                                        r.push({
                                            type: 'text',
                                            value: ', ',
                                        });
                                    }
                                    return [...acc, ...r];
                                }, []),
                                { type: 'text', value: ']' },
                            ],
                        },
                    ]
                    : [];
            }
            case 'stringLiteral': {
                return [
                    {
                        type: 'text',
                        value: `'${p.value}'`,
                    },
                ];
            }
            case 'intrinsic': {
                return [
                    {
                        type: 'text',
                        value: p.name,
                    },
                ];
            }
            default:
                console.log('unknown type: ', p.type);
                return [];
        }
    };
    const extractTSType = (node, fileName) => {
        const result = [];
        result.push({
            type: 'heading',
            depth: 2,
            children: [{ type: 'text', value: node.name }],
        });
        if (node.comment && node.comment.shortText) {
            result.push({
                type: 'paragraph',
                children: [
                    {
                        type: 'text',
                        value: node.comment.shortText,
                    },
                ],
            });
        }
        if (!repoNames[fileName]) {
            repoNames[fileName] = getRepoPath(path__default['default'].dirname(path__default['default'].resolve(fileName)));
            if (repoNames[fileName].filePath) {
                repoNames[fileName].packageName = JSON.parse(fs__default['default'].readFileSync(repoNames[fileName].filePath || '', 'utf8')).name;
                repoNames[fileName].relativePath = path__default['default'].relative(path__default['default'].dirname(repoNames[fileName].filePath || './'), fileName);
            }
        }
        if (repoNames[fileName]) {
            const { repo, relativePath, packageName } = repoNames[fileName];
            if (!repo) {
                console.log(fileName, repoNames);
            }
            const source = node.sources && node.sources.length && node.sources[0];
            const { line } = source || {};
            if (line) {
                const sourceLocation = fileName.includes('node_modules')
                    ? repo
                    : `${repo}/${relativePath}#L${line}`;
                result.push({
                    type: 'paragraph',
                    children: [
                        {
                            type: 'emphasis',
                            children: [
                                {
                                    type: 'text',
                                    value: 'defined in ',
                                },
                                {
                                    type: 'link',
                                    url: sourceLocation,
                                    children: [
                                        {
                                            type: 'text',
                                            value: `${packageName}/${relativePath}`,
                                        },
                                    ],
                                },
                            ],
                        },
                    ],
                });
            }
        }
        switch (node.kindString) {
            case 'Type alias':
                result.push(...extractPropType(node.type, true));
                break;
            case 'Type literal':
                node.children.forEach((child) => {
                    result.push(...extractPropType(child, true));
                });
                break;
            case 'Class':
            case 'Interface':
            default: {
                result.push(...extractInterface(node));
                break;
            }
            case 'Function': {
                result.push(...extractFunction(node));
                break;
            }
        }
        return result;
    };
    const project = app.convert(app.expandInputFiles(files));
    // Project may not have converted correctly
    if (project) {
        const fName = 'tmp-out-typedoc.json';
        // Alternatively generate JSON output
        try {
            app.generateJson(project, fName);
            const content = JSON.parse(fs__default['default'].readFileSync(fName, 'utf8'));
            const result = [];
            for (const entry of entries) {
                const main = content.children &&
                    content.children.find((child) => child.originalName === entry);
                if (main && main.children) {
                    main.children.forEach((child) => {
                        if (child.sources) {
                            const nodes = extractTSType(child, main.originalName);
                            result.push(...nodes);
                        }
                    });
                }
            }
            while (unresolvedTypeNames.length > 0) {
                const propName = unresolvedTypeNames[0];
                if (addedTypeNames.indexOf(propName) === -1) {
                    let propNode;
                    let fileName;
                    for (const child of content.children) {
                        if (child.children) {
                            propNode = child.children.find((c) => c.name === propName);
                            if (propNode) {
                                fileName = child.originalName;
                                break;
                            }
                        }
                    }
                    if (propNode) {
                        if (propNode.sources) {
                            const nodes = extractTSType(propNode, fileName);
                            result.push(...nodes);
                        }
                    }
                    else {
                        console.log('could not find external reference: ', propName);
                    }
                }
                unresolvedTypeNames.splice(0, 1);
            }
            // console.log(JSON.stringify(result, null, 2));
            return result;
        }
        finally {
            fs__default['default'].unlinkSync(fName);
        }
    }
    else {
        console.error('could not compile project');
    }
    return undefined;
};

/* eslint-disable prefer-spread */
const insertTSDoc = () => {
    const resolve = (file) => {
        if (file.startsWith('.')) {
            return path__default['default'].join(path__default['default'].resolve('./'), file);
        }
        return require.resolve(file);
    };
    return (node) => {
        const sections = extractCustomTag(node, 'tsdoc-typescript');
        if (sections) {
            sections.forEach(({ attrs, attributes }) => {
                if (attributes) {
                    const newNodes = [];
                    const files = attributes.find(attribute => attribute[0] === 'files');
                    const mappings = attributes.find(attribute => attribute[0] === 'map') || ['', ''];
                    const linkMaps = mappings[1].split(',').reduce((acc, l) => {
                        const s = l.split('|');
                        if (s.length === 2) {
                            return Object.assign(Object.assign({}, acc), { [s[0].trim()]: s[1].trim() });
                        }
                        return acc;
                    }, {});
                    const entry = attributes.find(attribute => attribute[0] === 'entry');
                    if (entry) {
                        const fileNames = files ? files[1].split(',').filter(s => s) : [];
                        const entryNames = entry ? entry[1].split(',').filter(s => s) : [];
                        fileNames.push(...entryNames);
                        const tsNodes = extractTSDoc(fileNames.map(file => resolve(file)), entryNames.map(file => resolve(file)), linkMaps);
                        if (tsNodes) {
                            newNodes.push(...tsNodes);
                        }
                        // console.log(tsdocs);
                    }
                    inlineNewContent(attrs, newNodes);
                }
            });
        }
    };
};

const propsToMDNodes = (propTable, repoFileName) => {
    const nodes = [];
    nodes.push({
        type: 'heading',
        depth: 2,
        children: [
            { type: 'html', value: '<ins>' },
            { type: 'text', value: propTable.displayName },
            { type: 'html', value: '</ins>' },
        ],
    });
    remark__default['default']()
        .use(() => (node) => {
        if (node.children) {
            nodes.push(...node.children);
        }
    })
        .process(propTable.description);
    nodes.push({
        type: 'paragraph',
        children: [
            {
                type: 'emphasis',
                children: [
                    { type: 'text', value: propTable.displayName },
                    { type: 'text', value: ' ' },
                    {
                        type: 'link',
                        url: repoFileName,
                        children: [{ type: 'text', value: 'source code' }],
                    },
                ],
            },
        ],
    });
    if (Object.keys(propTable.props).length) {
        const props = Object.keys(propTable.props).map(propName => {
            const prop = propTable.props[propName];
            return {
                name: propName,
                isOptional: !prop.required,
                type: [
                    {
                        type: 'emphasis',
                        children: [
                            { type: 'text', value: prop.type.raw || prop.type.name },
                        ],
                    },
                ],
                description: prop.description,
            };
        });
        const { propsTable } = createPropsTable('properties', props);
        // eslint-disable-next-line prefer-spread
        nodes.push(...propsTable);
    }
    return nodes;
};
const insertReactDocgenTypescript = (settings = { path: './src' }) => {
    return (node) => {
        const { propFilter = (prop) => {
            // Currently not working, prop.parent is always null.
            if (prop.parent) {
                return !prop.parent.fileName.includes('node_modules');
            }
            return true;
        }, componentNameResolver, shouldExtractLiteralValuesFromEnum = true, savePropValueAsString, } = settings;
        const parserOptions = {
            propFilter,
            componentNameResolver,
            shouldExtractLiteralValuesFromEnum,
            savePropValueAsString,
        };
        const parser = reactDocgenTypescript.withDefaultConfig(parserOptions);
        const sections = extractCustomTag(node, 'react-docgen-typescript');
        if (sections) {
            sections.forEach(({ attrs, attributes }) => {
                const newNodes = [];
                traverseDirs(attributes, (name, fileName, repoFolder) => {
                    const propTables = parser.parse(fileName);
                    if (propTables && propTables.length > 0) {
                        propTables.forEach(propTable => {
                            const propNodes = propsToMDNodes(propTable, `${repoFolder}/${name}`);
                            // eslint-disable-next-line prefer-spread
                            newNodes.push(...propNodes);
                        });
                    }
                });
                inlineNewContent(attrs, newNodes);
            });
        }
    };
};

var index = () => {
    const options = yargs.usage('ts-md -f <file>')
        .option('f', {
        alias: 'file',
        describe: 'Input file name',
        type: 'string',
    })
        .option('t', {
        alias: 'toc',
        describe: 'Table of content',
        type: 'boolean',
    }).argv;
    const fileName = options.file || 'README.md';
    const useToc = typeof options.toc !== 'undefined' ? options.toc : true;
    console.log('processing file:', chalk.red(path.resolve(fileName)));
    let r = remark__default['default']()
        .use(insertReactDocgenTypescript)
        .use(insertTSDoc)
        .use(insertOverview);
    if (useToc) {
        r = r.use(toc__default['default'], { tight: true });
    }
    r.process(fs.readFileSync(fileName, 'utf8'), function (err, file) {
        if (err) {
            throw err;
        }
        fs.writeFileSync(fileName, String(file));
    });
};

module.exports = index;
